## $Id: browser.py 8711 2012-06-13 13:58:05Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for basic applicants and related components.
"""
import grok
from zope.component import getUtility
from zope.i18n import translate
from waeup.kofa.widgets.datewidget import FriendlyDatetimeDisplayWidget
from waeup.kofa.students.interfaces import IStudentsUtils
from waeup.kofa.applicants.interfaces import (
    IApplicantRegisterUpdate, IApplicant, IApplicantEdit)
from waeup.kofa.applicants.browser import (ApplicantDisplayFormPage,
    ExportPDFPage,
    ApplicantManageFormPage, ApplicantEditFormPage,
    ApplicantRegistrationPage, ApplicantAddFormPage,
    OnlinePaymentDisplayFormPage, ApplicationFeePaymentAddPage,
    OnlinePaymentBreadcrumb, ExportPDFPaymentSlipPage,
    ApplicantBaseDisplayFormPage)
from waeup.kofa.applicants.viewlets import (
    PaymentReceiptActionButton)
from waeup.kofa.applicants.pdf import PDFApplicationSlip
from waeup.aaue.applicants.interfaces import (
    IPGApplicant, IUGApplicant, IPGApplicantEdit, IUGApplicantEdit,
    ICustomApplicantOnlinePayment, IPUTMEApplicantEdit,
    UG_OMIT_DISPLAY_FIELDS, PG_OMIT_DISPLAY_FIELDS,
    UG_OMIT_PDF_FIELDS, PG_OMIT_PDF_FIELDS,
    UG_OMIT_MANAGE_FIELDS, PG_OMIT_MANAGE_FIELDS,
    UG_OMIT_EDIT_FIELDS, PG_OMIT_EDIT_FIELDS, PUTME_OMIT_EDIT_FIELDS)
from waeup.aaue.interfaces import MessageFactory as _

#class RequestCallbackActionButton(RequestCallbackActionButton):
#    """ Display the base package callback button in custom pages.
#    """
#    grok.context(ICustomApplicantOnlinePayment)

#class CustomOnlinePaymentCallbackPage(OnlinePaymentCallbackPage):
#    """ Activate callback simulation view
#    """
#    grok.context(ICustomApplicantOnlinePayment)

class CustomOnlinePaymentBreadcrumb(OnlinePaymentBreadcrumb):
    """A breadcrumb for payments.
    """
    grok.context(ICustomApplicantOnlinePayment)

class PaymentReceiptActionButton(PaymentReceiptActionButton):
    grok.order(4)
    grok.context(ICustomApplicantOnlinePayment)

class CustomPDFApplicationSlip(PDFApplicationSlip):

    note = _(u'<br /><br /><br />'
              'Comfirm your exam venue 72 hours to the exam.')

    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicant)
            for field in PG_OMIT_PDF_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(IUGApplicant)
            for field in UG_OMIT_PDF_FIELDS:
                form_fields = form_fields.omit(field)
        return form_fields

class CustomApplicantDisplayFormPage(ApplicantDisplayFormPage):
    """A display view for applicant data.
    """

    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicant)
            for field in PG_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(IUGApplicant)
            for field in UG_OMIT_DISPLAY_FIELDS:
                form_fields = form_fields.omit(field)
        return form_fields

class CustomApplicantManageFormPage(ApplicantManageFormPage):
    """A full edit view for applicant data.
    """
    
    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicant)
            for field in PG_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(IUGApplicant)
            for field in UG_OMIT_MANAGE_FIELDS:
                form_fields = form_fields.omit(field)
        form_fields['student_id'].for_display = True
        form_fields['applicant_id'].for_display = True
        return form_fields

class CustomApplicantEditFormPage(ApplicantEditFormPage):
    """An applicant-centered edit view for applicant data.
    """

    @property
    def form_fields(self):
        target = getattr(self.context.__parent__, 'prefix', None)
        if target is not None and target.startswith('pg'):
            form_fields = grok.AutoFields(IPGApplicantEdit)
            for field in PG_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        elif target is not None and target.startswith('putme'):
            form_fields = grok.AutoFields(IPUTMEApplicantEdit)
            for field in PUTME_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        else:
            form_fields = grok.AutoFields(IUGApplicantEdit)
            for field in UG_OMIT_EDIT_FIELDS:
                form_fields = form_fields.omit(field)
        form_fields['applicant_id'].for_display = True
        form_fields['reg_number'].for_display = True
        return form_fields

class CustomOnlinePaymentDisplayFormPage(OnlinePaymentDisplayFormPage):
    """ Page to view an online payment ticket
    """
    grok.context(ICustomApplicantOnlinePayment)
    form_fields = grok.AutoFields(ICustomApplicantOnlinePayment).omit('ac')
    form_fields[
        'creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields[
        'payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    #grok.template('payment_view')

    #@property
    #def transaction_code(self):
    #    tcode = self.context.p_id
    #    return tcode[len(tcode)-8:len(tcode)]

class CustomApplicationFeePaymentAddPage(ApplicationFeePaymentAddPage):
    """ Page to add an online payment ticket
    """
    factory = u'waeup.CustomApplicantOnlinePayment'

class CustomExportPDFPaymentSlipPage(ExportPDFPaymentSlipPage):
    """Deliver a PDF slip of the context.
    """
    grok.context(ICustomApplicantOnlinePayment)
    form_fields = grok.AutoFields(ICustomApplicantOnlinePayment).omit('ac')
    form_fields['creation_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')
    form_fields['payment_date'].custom_widget = FriendlyDatetimeDisplayWidget('le')

#    @property
#    def note(self):
#        if self.context.p_state == 'paid':
#            return None
#        tcode = self.context.p_id
#        tcode = tcode[len(tcode)-8:len(tcode)]
#        amount = self.context.amount_auth
#        note = translate(_(
#            u"""<br /><br /><br />
#The tranzaction code is <strong>${a}</strong>.""",
#            mapping = {'a':tcode}))
#        return note
